const https = require('https')

/**
 * Connection with KSC Open API server
 * Connection based on TCP socket
 * This connection is maintained permanently
 */
class Connection {
  /**
   * Create new connection to KSC Open API
   * @param {string} openAPIHost - ip or domain name of KSC Open API server
   * @param {number} openAPIPort - port for KSC Open API server
   * @param {string} openAPIVersion - version of KSC Open API server
   * @param {Function} handshakeFunc - handshake method (login or other) for KSC Open API server
   */
  constructor ({ openAPIHost, openAPIPort, openAPIVersion, handshakeFunc }) {
    this.uid = null
    this.kscUserId = null
    this.config = {
      openAPIHost,
      openAPIPort,
      openAPIVersion,
      handshakeFunc
    }
    this.socket = https.globalAgent.createConnection({
      host: this.config.openAPIHost,
      port: this.config.openAPIPort
    })
    this.isBusy = false
    this.queue = []
    this.eventHandlers = []
  }

  /**
    * Open permanent TCP connection with Open API server
    * @return {Promise}
    */
  async open () {
    const { response, body } = await this.config.handshakeFunc(this)
    if (response.statusCode !== 200) {
      throw new Error(body)
    }
  }

  /**
   * Register callback which fire on closing TCP socket`s event
   * @param {string} eventName - TCP socket`s event
   * @param {function} callback
   */
  on (eventName, callback) {
    if (!Array.isArray(this.eventHandlers[eventName])) {
      this.eventHandlers[eventName] = []
    }
    this.eventHandlers[eventName].push(callback)
    this.socket.on(eventName, callback)
  }

  /**
   * Check is tcp connection event (close, error, etc.) handled
   * @param {string} eventName - TCP socket`s event
   */
  isEventHandled (eventName) {
    const eventHandlers = this.eventHandlers[ eventName ]
    return Array.isArray(eventHandlers) && eventHandlers.length > 0
  }

  /**
   * Close connection
   */
  close () {
    try {
      this.socket.destroy('TCP socket closed from Node.js Open API client')
    } catch (error) {
      console.error(error)
    }
  }

  /**
   * Set KSC user id to connection
   * @param kscUserId
   */
  setKscUserId (kscUserId) {
    this.kscUserId = kscUserId
  }
}

module.exports = Connection
