const _ = require('lodash')
const BaseOpenAPIClient = require('../base-open-api-client')
const className = 'Policy'

module.exports = class Policy extends BaseOpenAPIClient {
  /**
   * @description Returns policies located in specified group.
   * @param {number} nGroupId - group identifier, -1 for all
   * @param {Connection} connection
   * @return {Promise}
   */
  GetPoliciesForGroup ({ nGroupId }, connection) {
    return this.baseRequest({
      command: `${className}.GetPoliciesForGroup`,
      data: { nGroupId },
      connection
    })
      .then(this.deserialize)
  }

  /**
   * Get policy general data
   * @param {number} nPolicy - policy identifier
   * @param {Connection} connection
   * @return {Promise.<TResult>}
   * @constructor
   */
  GetPolicyData ({ nPolicy }, connection) {
    nPolicy = parseInt(nPolicy)
    return this.baseRequest({ command: `${className}.GetPolicyData`, data: { nPolicy }, connection })
      .then(this.deserialize)
  }

  /**
   * Return identifier of opened SsContents, must be closed with SsContents::SS_Release
   * @param {number} nPolicy - policy identifier
   * @param {number} nRevisionId - policy revision id, 0 means 'current policy'
   * @param {number} nLifeTime - timeout in milliseconds to keep this SsContents object alive, zero means 'default
   *     value'
   */
  GetPolicyContents ({ nPolicy, nRevisionId = 0, nLifeTime = 0 } = {}, connection) {
    return this.baseRequest({ command: `${className}.GetPolicyContents`, data: { nPolicy, nRevisionId, nLifeTime }, connection })
      .then(this.getResult)
  }

  /**
   * Create new policy
   * @param {string} KLPOL_DN - policy name
   * @param {string} KLPOL_PRODUCT - policy product alias
   * @param {string} KLPOL_VERSION - product version
   * @param {number} KLPOL_GROUP_ID - parent group id
   * @param {boolean} KLPOL_ACTIVE - policy activity
   * @param {boolean} KLPOL_ROAMING - policy out-of-office flag
   * @param {boolean} KLPOL_ACCEPT_PARENT - must be modified by parent policy
   * @param {boolean} KLPOL_FORCE_DISTRIB2CHILDREN - must modify child policies
   * @param {Connection} connection
   * @return {Promise.<TResult>}
   * @constructor
   */
  AddPolicy ({
    KLPOL_DN, KLPOL_PRODUCT, KLPOL_VERSION, KLPOL_GROUP_ID, KLPOL_ACTIVE = false,
    KLPOL_ROAMING = false, KLPOL_ACCEPT_PARENT = true, KLPOL_FORCE_DISTRIB2CHILDREN = false
  } = {}, connection) {
    return this.baseRequest({
      command: `${className}.AddPolicy`,
      data: {
        pPolicyData: {
          KLPOL_DN,
          KLPOL_PRODUCT,
          KLPOL_VERSION,
          KLPOL_GROUP_ID,
          KLPOL_ACTIVE,
          KLPOL_ROAMING,
          KLPOL_ACCEPT_PARENT,
          KLPOL_FORCE_DISTRIB2CHILDREN
        }
      },
      connection
    })
      .then(this.getResult)
  }

  /**
   * Update policy attributes
   */
  UpdatePolicyData ({
    nPolicy,
    pPolicyData: { KLPOL_DN, KLPOL_ACTIVE, KLPOL_ROAMING, KLPOL_ACCEPT_PARENT, KLPOL_FORCE_DISTRIB2CHILDREN }
  } = {},
  connection) {
    return this.baseRequest({
      command: `${className}.UpdatePolicyData`,
      data: {
        nPolicy,
        pPolicyData: {
          KLPOL_DN, KLPOL_ACTIVE, KLPOL_ROAMING, KLPOL_ACCEPT_PARENT, KLPOL_FORCE_DISTRIB2CHILDREN
        }
      },
      connection
    })
      .then(this.getBody)
  }

  /**
   * Copies the specified policy and optionally deletes it
   * @param {number} nPolicy - policy id
   * @param {number} nNewGroupId - group to copy policy
   * @param {boolean} bDeleteOrigin - if server must delete nPolicy after copying
   * @param {Object} pExtraData - optional container with extra attributes, may contain following data: KLPOL_DN, KLPOL_ACTIVE
   * @param {Connection} connection
   * @return {Promise.<TResult>}
   * @constructor
   */
  CopyOrMovePolicy ({ nPolicy, nNewGroupId, bDeleteOrigin = false, pExtraData = null } = {}, connection) {
    return this.baseRequest({
      command: `${className}.CopyOrMovePolicy`,
      data: {
        nPolicy, nNewGroupId, bDeleteOrigin, pExtraData
      },
      connection
    })
      .then(this.getResult)
  }

  /**
   * @description Delete policy by id
   * @param {number} nPolicy - policy identifier
   * @param {Connection} connection
   * @return {Promise.<TResult>}
   * @constructor
   */
  DeletePolicy ({ nPolicy }, connection) {
    return this.baseRequest({ command: `${className}.DeletePolicy`, data: { nPolicy }, connection })
      .then(this.getBody)
  }

  /**
   * @description Returns active and roaming policies that affect specified group
   * @param {number} nGroupId - group identifier, -1 for all
   * @param {Connection} connection
   * @return {Promise}
   */
  GetEffectivePoliciesForGroup ({ nGroupId }, connection) {
    return this.baseRequest({
      command: `${className}.GetEffectivePoliciesForGroup`,
      data: { nGroupId },
      connection
    })
      .then(this.deserialize)
  }

  /**
   * @description Revert policy to
   * Replaces the specified policy nPolicy by its revision (older version) nRevisionId
   * @param {number} nPolicyid of policy to revert
   * @param {number} nRevisionId id of policy revision
   * @param {Connection} connection
   * @return {Promise}
   */
  RevertPolicyToRevision ({ nPolicy, nRevisionId }, connection) {
    return this.baseRequest({
      command: `${className}.RevertPolicyToRevision`,
      data: { nPolicy, nRevisionId },
      connection
    }).then(this.getBody)
  }

  /**
   *
   * @description Acquire array of outbreak policies.
   * @param {*} { args } Empty param
   * @param {*} connection
   * @returns {Promise<Array>} Container with attribute KLPOL_POL_OUTBREAK of type paramArray, each entry contains two attributes KLPOL_ID and KLPOL_OUTBREAK_MASK
   */
  GetOutbreakPolicies ({ args }, connection) {
    return this.baseRequest({
      command: `${className}.GetOutbreakPolicies`,
      connection
    }).then(this.deserialize)
  }

  /**
   *
   * @description Specify array of outbreak policies.
   * @param {Array} { pData } contains attribute KLPOL_POL_OUTBREAK of type paramArray, each entry contains two attributes KLPOL_ID and KLPOL_OUTBREAK_MASK
   * @param {*} connection
   * @returns {Promise}
   */
  SetOutbreakPolicies ({ pData }, connection) {
    return this.baseRequest({
      command: `${className}.SetOutbreakPolicies`,
      data: { pData },
      connection
    }).then(this.getBody)
  }
}
