const baseOpenAPIClient = require('../base-open-api-client')
const className = 'SrvView'

module.exports = class SrvView extends baseOpenAPIClient {
  /**
   * Finds data records for srvview wstrViewName that satisfy conditions from filter string wstrFilter
   * and creates a server-side collection of found data records
   * @param {string} wstrViewName - name of srvview
   * @param {string} wstrFilter - filter string, contains a condition over srvview attributes
   * @param {Array} vecFieldsToReturn - array of srvview attribute names to return
   * @param {Array} vecFieldsToOrder - array of containers each of them containing two attributes
   * @param {Object} pParams - extra options
   * @param {number} lifetimeSec - max result-set lifetime in seconds
   * @param {Connection} connection
   * @constructor
   */
  ResetIterator ({ wstrViewName, wstrFilter, vecFieldsToReturn, vecFieldsToOrder = [], pParams = {}, lifetimeSec = 1000 }, connection) {
    return this.baseRequest({
      command: `${className}.ResetIterator`,
      data: {
        wstrViewName,
        wstrFilter,
        vecFieldsToReturn,
        vecFieldsToOrder,
        pParams,
        lifetimeSec
      },
      connection
    }).then(body => this.getResult(body, 'wstrIteratorId'))
  }

  /**
   * Returns number of elements contained in the specified result-set
   * @param {string} wstrIteratorId - result-set ID, identifier of the server-side ordered collection of found data records
   * @param {Connection} connection
   * @return {Promise.<TResult>}
   * @constructor
   */
  GetRecordCount ({ wstrIteratorId }, connection) {
    return this.baseRequest({ command: `${className}.GetRecordCount`, data: { wstrIteratorId }, connection })
      .then(this.getResult)
  }

  /**
   * Returns number of elements contained in the specified result-set
   * @param {string} wstrIteratorId - result-set ID, identifier of the server-side ordered collection of found data records
   * @param {number} nStart - zero-based start position
   * @param {number} nEnd - zero-based finish position
   * @param {Connection} connection
   * @return {Promise.<TResult>}
   * @constructor
   */
  GetRecordRange ({ wstrIteratorId, nStart, nEnd }, connection) {
    return this.baseRequest({ command: `${className}.GetRecordRange`, data: { wstrIteratorId, nStart, nEnd }, connection })
      .then(body => this.deserialize(body, 'pRecords'))
  }

  /**
   * Releases the specified result-set and frees associated memory
   * @param {string} wstrIteratorId - result-set ID, identifier of the server-side ordered collection of found data records
   * @param {Connection} connection
   * @return {Promise.<TResult>}
   * @constructor
   */
  ReleaseIterator ({ wstrIteratorId }, connection) {
    return this.baseRequest({ command: `${className}.ReleaseIterator`, data: { wstrIteratorId }, connection })
      .then(this.getBody)
  }

  /**
   * common fetch method (wrapper function for methods of these class)
   * @param {string} wstrViewName - name of srvview
   * @param {string} wstrFilter - filter string, contains a condition over srvview attributes
   * @param {Array} vecFieldsToReturn - array of srvview attribute names to return
   * @param {Array} vecFieldsToOrder - array of containers each of them containing two attributes
   * @param {Object} pParams - extra options
   * @param {number} lifetimeSec - max result-set lifetime in seconds
   * @param {number} nStart - zero-based start position
   * @param {number} nEnd - zero-based finish position
   * @param {string} all - get all records
   */
  Fetch ({
    wstrViewName,
    wstrFilter,
    vecFieldsToReturn,
    vecFieldsToOrder = [],
    pParams = null,
    lifetimeSec = 1000,
    nStart = 0,
    nEnd,
    all = true
  }, connection) {
    return this.ResetIterator({
      wstrViewName,
      wstrFilter,
      vecFieldsToReturn,
      vecFieldsToOrder: this.serialize(vecFieldsToOrder),
      pParams,
      lifetimeSec
    }, connection)
      .then(iteratorContainer => {
        const wstrIteratorId = iteratorContainer.result
        return this.GetRecordCount({ wstrIteratorId }, connection)
          .then(countContainer => {
            const count = countContainer.result
            const params = {
              wstrIteratorId,
              nStart: all ? 0 : nStart,
              nEnd: all ? count : nEnd || count
            }
            return this.GetRecordRange(params, connection)
          })
          .then(responseContainer => {
            return this.ReleaseIterator({ wstrIteratorId }, connection)
              .then(() => responseContainer)
          })
      })
  }
}
