const _ = require('lodash')
const BaseOpenAPIClient = require('../base-open-api-client')
const className = 'Tasks'

module.exports = class Tasks extends BaseOpenAPIClient {
  /**
   * Проверяет задачу на соответствие группе тестов.
   * Если тестовое значение - строка, то она ищется как
   * подстрока проверяемой задачи, иначе проверяется
   * полное соответствие.
   *
   * @param {Object} item - Проверяемая задача
   * @param {Object} tests - Набор тестов ({поле: значение})
   */
  testItem (item, tests) {
    for (let field in tests) {
      const searchValue = tests[field]
      const itemValue = _.get(item, field)
      if (itemValue !== undefined && typeof searchValue === 'string') {
        if (!~itemValue.toLowerCase().indexOf(searchValue.toLowerCase())) {
          return false
        }
      } else {
        if (itemValue !== searchValue) {
          return false
        }
      }
    }
    return true
  }

  ResetTasksIterator ({ nGroupId, bGroupIdSignificant = true, bIncludeSupergroups = true }, connection) {
    return this.baseRequest({
      command: `${className}.ResetTasksIterator`,
      data: {
        nGroupId,
        bGroupIdSignificant,
        bIncludeSupergroups
      },
      connection
    })
      .then(body => this.getResult(body, 'strTaskIteratorId'))
  }

  /**
   * Получение следующей задачи из выборки, основанной на итераторе
   * @param {string} strTaskIteratorId - идентификатор итератора
   * @param {boolean} deserialize - флаг, нужна ли десериализация данных
   * @param {Connection} connection
   * @return {Promise.<*>}
   */
  async getNextTask ({strTaskIteratorId, deserialize = false}, connection) {
    const result = await this.baseRequest({
      command: `${className}.GetNextTask`,
      data: {strTaskIteratorId},
      connection
    })
    const resultHandler = deserialize ? this.deserialize : this.getResult
    return resultHandler(result, 'pTaskData')
  }

  /**
   * Start specified task
   * @param {string} strTask - task identifier
   * @param {Connection} connection
   * @return {Promise.<TResult>}
   * @constructor
   */
  RunTask ({ strTask }, connection) {
    return this.baseRequest({ command: `${className}.RunTask`, data: { strTask }, connection })
      .then(this.getBody)
  }

  /**
   * Suspend execution of the specified task
   * @param {string} strTask - task identifier
   * @param {Connection} connection
   * @return {Promise.<TResult>}
   * @constructor
   */
  SuspendTask ({ strTask }, connection) {
    return this.baseRequest({ command: `${className}.SuspendTask`, data: { strTask }, connection })
      .then(this.getBody)
  }

  /**
   * Resume specified task
   * @param {string} strTask - task identifier
   * @param {Connection} connection
   * @return {Promise.<TResult>}
   * @constructor
   */
  ResumeTask ({ strTask }, connection) {
    return this.baseRequest({ command: `${className}.ResumeTask`, data: { strTask }, connection })
      .then(this.getBody)
  }

  /**
   * Cancel execution of the specified task
   * @param {string} strTask - task identifier
   * @param {Connection} connection
   * @return {Promise.<TResult>}
   * @constructor
   */
  CancelTask ({ strTask }, connection) {
    return this.baseRequest({ command: `${className}.CancelTask`, data: { strTask }, connection })
      .then(this.getBody)
  }

  /**
   * Delete the specified task
   * @param {string} strTask - task identifier
   * @param {Connection} connection
   * @constructor
   */
  DeleteTask ({ strTask }, connection) {
    return this.baseRequest({ command: `${className}.DeleteTask`, data: { strTask }, connection })
      .then(this.getBody)
  }

  GetTaskData ({taskId, skipDeserialization, strSrvObjId}, connection) {
    let strTask = taskId && taskId.toString()
    return this.baseRequest({ command: `${className}.GetTaskData`, data: { strTask, strSrvObjId }, connection })
      .then(skipDeserialization ? this.getBody : this.deserialize)
  }

  AddTask ({taskData, skipSerialization}, connection) {
    let pData = skipSerialization ? taskData : this.serialize(taskData)
    return this.baseRequest({
      command: `${className}.AddTask`,
      data: { pData },
      connection })
      .then(this.getBody)
  }
  UpdateTask ({taskId, taskData, skipSerialization, strSrvObjId}, connection) {
    let strTask = taskId && taskId.toString()
    let pData = skipSerialization ? taskData : this.serialize(taskData)
    return this.baseRequest({
      command: `${className}.UpdateTask`,
      data: {
        strTask,
        pData,
        strSrvObjId
      },
      connection })
      .then(this.getBody)
  }

  /**
   * Получение конечного списка задач для группы администрирования
   * @param {number} hostGroupId - идентификатор группы администрирования
   * @param {number} tasksCount - максимальное количество задач, которое необходимо получить. Ноль — получить все задачи
   * @param {Object} searchData - данные для фильтрации списка
   * @param {boolean} deserialize - флаг, нужна ли десериализация данных
   * @param {Connection} connection
   * @return {Promise.<{result}>}
   * @constructor
   */
  async GetTasks ({ hostGroupId, tasksCount, searchData, bGroupIdSignificant, bIncludeSupergroups, deserialize = false }, connection) {
    const { result: iterator } = await this.ResetTasksIterator({
      nGroupId: parseInt(hostGroupId),
      bGroupIdSignificant,
      bIncludeSupergroups
    }, connection)
    const tasks = await this.DrainTasksRecursive({
      iterator,
      tasksCount,
      searchData,
      deserialize
    }, connection)
    this.ReleaseTasksIterator(iterator, connection)
    return {
      result: tasks
    }
  }
  ResetHostIteratorForTaskStatus ({strTask, pFields2Return, nHostStateMask, nLifetime}, connection) {
    return this.baseRequest({
      command: `${className}.ResetHostIteratorForTaskStatus`,
      data: {strTask, pFields2Return, nHostStateMask, nLifetime},
      connection
    }).then(body => this.getResult(body, 'strHostIteratorId'))
  }
  GetHostStatusRecordRange ({strHostIteratorId, nStart, nEnd}, connection) {
    return this.baseRequest({
      command: `${className}.GetHostStatusRecordRange`,
      data: {strHostIteratorId, nStart, nEnd},
      connection
    }).then(body => this.getResult(body, 'pParHostStatus'))
  }
  ReleaseHostStatusIterator (strHostIteratorId, connection) {
    return this.baseRequest({
      command: `${className}.ReleaseHostStatusIterator`,
      data: {strHostIteratorId},
      connection
    }).then(this.getBody)
  }
  GetHostStatusRecordsCount (strHostIteratorId, connection) {
    return this.baseRequest({
      command: `${className}.GetHostStatusRecordsCount`,
      data: {strHostIteratorId},
      connection
    }).then(this.getBody)
  }
  async GetTaskHistory ({taskId, options, entitiesAmount = 50}, connection) {
    let pFields2Return = [
      'hostdn',
      'group_name',
      'task_new_state',
      'KLVSRV_DN',
      'rise_time',
      'GNRL_EA_DESCRIPTION'
    ]
    const defaults = {
      pFields2Return,
      strHostName: '',
      pFilter: {type: 'params', value: {}},
      pSortFields: []
    }
    let data = _.assign(options || defaults, {strTask: taskId.toString()})
    if (data.pSortFields.length) {
      data.pSortFields = this.serialize(data.pSortFields)
    }
    let iteratorString = await this.baseRequest({command: `${className}.GetTaskHistory`, data, connection})
                             .then(res => this.getBody(res).result.strIteratorId)
    const OpenAPI = require('../open-api-client')
    let taskResults = await OpenAPI.EventProcessing.GetRecordRange({
      strIteratorId: iteratorString,
      nStart: 0,
      nEnd: entitiesAmount
    }, connection)/* .then(this.getBody) */
    return {result: taskResults.result.KLEVP_EVENT_RANGE_ARRAY}
  }
  async GetTaskResults ({taskId, options, entitiesAmount = 50}, connection) {
    let statuses = {
      'Pending': 0x01,
      'Running': 0x02,
      'Finished successfully': 0x04,
      'Finished with warning': 0x08,
      'Failed': 0x10,
      'Scheduled': 0x20,
      'Paused': 0x40
    }
    let fields = [
      'state_descr',
      'domain_name',
      'hostname',
      'hostdn',
      'state_code',
      'try_count',
      'start_time',
      'finish_time',
      'completed_percent'
    ]
    let forAllStatuses = Object.keys(statuses).reduce((mask, statusName) => mask | statuses[statusName], 0)
    let defaults = {
      pFields2Return: fields,
      nHostStateMask: forAllStatuses,
      nLifetime: 7200
    }
    let data = _.assign(options || defaults, {strTask: taskId.toString()})
    let iteratorString = (await this.ResetHostIteratorForTaskStatus(data, connection)).result
    const taskResults = await this.GetHostStatusRecordRange({
      strHostIteratorId: iteratorString,
      nStart: 0,
      nEnd: entitiesAmount
    }, connection)
    await this.ReleaseHostStatusIterator(iteratorString, connection)
    let count = await this.GetHostStatusRecordsCount(iteratorString, connection)
    console.log(count, 'STATUSES COUNT')
    return {result: taskResults.result.statuses}
  }
  async GetTaskStatus (taskId, connection) {
    const statusMapping = {
      1: 'Pending',
      2: 'Running',
      4: 'Finished Successfully',
      8: 'Finished with warning',
      16: 'Failed',
      32: 'Scheduled',
      64: 'Paused'
    }
    let { result: taskStatistics } = await this.GetTaskStatistics(taskId, connection)
    if (!taskStatistics) {
      return {}
    }
    for (let bitStatus in statusMapping) {
      const stringStatus = statusMapping[bitStatus]
      taskStatistics[stringStatus] = taskStatistics[bitStatus]
      delete taskStatistics[bitStatus]
    }
    return { result: taskStatistics }
  }

  async GetTasksStatuses (tasks, connection) {
    let tasksWithStatuses = []
    for (let i = 0; i < tasks.length; i++) {
      const task = tasks[i]
      const status = await this.GetTaskStatus(task.id.toString(), connection)
      task.status = status.result
      tasksWithStatuses.push(task)
    }

    return { result: tasksWithStatuses }
  }

  /**
   * Рекурсивное получение задач: one by one
   * @param {string} iterator - идентификатор итератора
   * @param {number} tasksCount - максимальное количество задач, которое необходимо получить. Ноль — получить все задачи
   * @param {Object} searchData - данные для фильтрации списка
   * @param {boolean} deserialize - флаг, нужна ли десериализация данных
   * @param {Connection} connection
   * @param {array} tasks
   * @return {Promise.<Array>}
   * @constructor
   */
  async DrainTasksRecursive ({ iterator, tasksCount = 0, searchData, deserialize = false }, connection, tasks = []) {
    const newTask = await this.getNextTask({
      strTaskIteratorId: iterator,
      deserialize
    }, connection)
    if (!newTask.result) {
      return tasks
    }
    const isInvisible = newTask.result &&
      newTask.result.TASK_ADDITIONAL_PARAMS &&
      newTask.result.TASK_ADDITIONAL_PARAMS.KLPRCI_TASK_INVISIBLE
    if ((!searchData || this.testItem(newTask.result, searchData)) && !isInvisible) {
      tasks.push(newTask.result)
    }
    let result
    if (tasksCount !== 0 && tasks.length >= tasksCount) {
      result = tasks
    } else {
      result = await this.DrainTasksRecursive({ iterator, tasksCount, searchData, deserialize }, connection, tasks)
    }
    return result
  }

  GetTaskStatistics (taskId, connection) {
    return this.baseRequest({
      command: `${className}.GetTaskStatistics`,
      data: {
        strTask: taskId
      },
      connection
    })
      .then(this.getResult)
  }

  GetNextHostStatus (iterator, connection) {
    return this.baseRequest({
      command: `${className}.GetNextHostStatus`,
      data: {
        strHostIteratorId: iterator,
        nCount: 1
      },
      connection
    })
      .then(this.getBody)
  }

  ReleaseTasksIterator (iterator, connection) {
    return this.baseRequest({
      command: `${className}.ReleaseTasksIterator`,
      data: {
        strTaskIteratorId: iterator
      },
      connection
    })
      .then(this.getBody)
  }

  ProtectPassword (strPassword, connection) {
    return this.baseRequest({
      command: `${className}.ProtectPassword`,
      data: {
        strPassword
      },
      connection,
      tranformRequestBodyLogs: body => {
        body.strPassword = '🙈🙉🙊'
        return body
      }
    })
      .then(this.getBody)
  }
}
