const { eventNames: { activity } } = require('@kl/constants')

const openAPIClient = require('./open-api-client')

module.exports = class OpenAPISocketProxy {
  constructor ({ getConnection }) {
    const self = this
    this.attachToSocket = attachToSocket

    function attachToSocket (socket) {
      socket.on('run-command', function (command, data) {
        self.onSocketMessage({ socket })
        onCommandStart(socket, command, data)
      })
      socket.on('open-api:get-interface', function () {
        self.onSocketMessage({ socket })
        socket.emit('open-api:announce-interface', openAPIClient.interface)
      })
    }

    function onCommandEnd (socket, command, resultData) {
      command.result = resultData
      command.result.itr = isIterator(command.name)
      socket.emit('command-end', command)
    }

    function onCommandError (socket, command, error) {
      console.error(error)
      command.error = error.message ? error.message : error
      command.errorLocalization = error.localization
      socket.emit('command-error', command)
    }

    function isIterator (cmdName) {
      var iteratorMethods = [
        'AdHosts.FindAdGroups',
        'AdHosts.GetChildOUs',
        'AdHosts.GetChildComputers',
        'HostGroup.FindHosts',
        'HostGroup.FindGroups',
        'HostGroup.FindUsers',
        'HostGroup.FindIncidents',
        'HstAccessControl.FindRoles',
        'Tasks_GetTaskHistory',
        'SecurityPolicy_FindRoles',
        'SrvView_GetChunkAccessor',
        'UserList_FindUsers',
        'UserDevices_FindDevices',
        'ScanDiapasons.GetDiapasons'
      ]
      return iteratorMethods.indexOf(cmdName) !== -1
    }

    async function runCommand (socket, command, data) {
      var successCallback = onCommandEnd.bind(this, socket, command)
      var errorCallback = onCommandError.bind(this, socket, command)

      var [openAPIClass, openAPIMethod] = command.name.split('.')

      if (!openAPIClient.interface[openAPIClass] || !openAPIClient.interface[openAPIClass][openAPIMethod]) {
        const error = `There is no ${openAPIClass}.${openAPIMethod} command in Open API interface`
        errorCallback(error)
        return
      };

      let connection = getConnection(socket)
      if (!connection) {
        onCloseConnection()
        return
      } else if (!connection.isEventHandled('error')) {
        connection.on('error', onCloseConnection)
      }

      try {
        const response = await openAPIClient[openAPIClass][openAPIMethod](data, connection)
        successCallback(response)
      } catch (error) {
        errorCallback(error)
      }

      function onCloseConnection () {
        console.log('API connection is lost. Emit socket logout.')
        socket.emit('logout')
      }
    }

    function onCommandStart (socket, command, data) {
      var cmdText = 'ChunkAccessor.'
      if (command.name === 'itr') {
        var start = data.start
        switch (data.action) {
          case 'next':
            cmdText += 'GetItemsChunk'
            break
          case 'prev':
            cmdText += 'GetItemsChunk'
            break
          case 'close':
            cmdText += 'Release'
            break
          case 'count':
            cmdText += 'GetItemsCount'
            break
        }
        command.name = cmdText
        runCommand(socket, command, {
          strAccessor: data.id,
          nStart: start,
          nCount: data.count
        })
      } else {
        runCommand(socket, command, data)
      }
    }
  }

  onSocketMessage ({ socket }) {
    try {
      runtime.trigger(activity.socket, {
        uid: socket.request.user.uid
      })
    } catch (error) {
      console.error(error)
    }
  }
}
