const BaseOpenAPIClient = require('../base-open-api-client')
const className = 'SecurityPolicy3'

/**
 * Allows to manage security groups of internal users.
 */
module.exports = class SecurityPolicy3 extends BaseOpenAPIClient {
  /**
   * @description Creates a security group on a server.
   * @param {object} pGrpParams - parameters of a group.
   *
   * There are possible values:
   *   KLSPL_SEC_GRP_NAME (paramString) - name of a group,
   *   KLSPL_SEC_GRP_DESC (paramString) - description of a group.
   *
   * @param {number} lVsId - id of a virtual server, a value <= 0 means main server
   * @param {Connection} connection
   * @return {Promise.<TResult>}
   */
  AddSecurityGroup ({ pGrpParams, lVsId = 0 }, connection) {
    return this.baseRequest({
      command: `${className}.AddSecurityGroup`,
      data: {
        pGrpParams,
        lVsId
      },
      connection
    })
      .then(this.getResult)
  }

  /**
   * @description Adds user into a security group.
   *
   * If a group or user does not exist a error occurs.
   * Connection to a virtual server has access only to groups and users
   *   which located on this virtual server.
   *
   * A user located on a virtual server can be added only into a group
   *   located on this virtual server, otherwise a error occurs.
   *
   * A user located on a main server can be added only into a group
   *   located on this main server, otherwise a error occurs.
   *
   * @param {number} lUserId - user id
   * @param {number} lGrpId - security group id
   * @param {Connection} connection
   * @return {Promise.<TResult>}
   */
  AddUserIntoSecurityGroup ({ lUserId, lGrpId }, connection) {
    return this.baseRequest({
      command: `${className}.AddUserIntoSecurityGroup`,
      data: {
        lUserId,
        lGrpId
      },
      connection
    })
      .then(this.getResult)
  }

  /**
   * @description Closes user connections.
   * @param {number} lUserId - user id
   * @param {Connection} connection
   * @return {Promise.<TResult>}
   */
  CloseUserConnections ({ lUserId }, connection) {
    return this.baseRequest({
      command: `${className}.CloseUserConnections`,
      data: {
        lUserId
      },
      connection
    })
      .then(this.getResult)
  }

  /**
   * @description Delete a security group.
   *
   * Connection to a virtual server has access only to
   *   groups which located on this virtual server.
   *
   * @param {number} lGrpId - security group id
   * @param {Connection} connection
   * @return {Promise.<TResult>}
   */
  DeleteSecurityGroup ({ lGrpId }, connection) {
    return this.baseRequest({
      command: `${className}.DeleteSecurityGroup`,
      data: { lGrpId },
      connection
    })
    .then(this.getResult)
  }

  /**
   * @description Removes user from a security group.
   *
   * Connection to a virtual server has access only to groups and users
   *   which located on this virtual server.
   *
   * @param {number} lGrpId - security group id
   * @param {number} lUserId - user id
   * @param {Connection} connection
   * @return {Promise.<TResult>}
   */
  DeleteUserFromSecurityGroup ({ lGrpId, lUserId }, connection) {
    return this.baseRequest({
      command: `${className}.DeleteUserFromSecurityGroup`,
      data: {
        lGrpId,
        lUserId
      },
      connection
    })
      .then(this.getResult)
  }

  /**
   * @description Moves user from one security group into other security group.
   * @param {number} lUserId - user id
   * @param {number} lGrpIdFrom - security group id
   * @param {number} lGrpIdTo - security group id
   * @param {Connection} connection
   * @return {Promise.<TResult>}
   */
  MoveUserIntoOtherSecurityGroup ({ lUserId, lGrpIdFrom, lGrpIdTo }, connection) {
    return this.baseRequest({
      command: `${className}.MoveUserIntoOtherSecurityGroup`,
      data: {
        lUserId,
        lGrpIdFrom,
        lGrpIdTo
      },
      connection
    })
      .then(this.getResult)
  }

  /**
   * @description Updates a security group.
   *
   * If a group does not exist a error occurs.
   * Connection to a virtual server has access only to groups
   *   which located on this virtual server.
   *
   * @param {number} lGrpId - security group id
   * @param {object} pGrpParams - parameters of a group.
   *
   * There are possible values:
   *   KLSPL_SEC_GRP_NAME (paramString) - name of a group,
   *   KLSPL_SEC_GRP_DESC (paramString) - description of a group.
   *
   * @param {Connection} connection
   * @return {Promise.<TResult>}
   */
  UpdateSecurityGroup ({ lGrpId, pGrpParams }, connection) {
    return this.baseRequest({
      command: `${className}.UpdateSecurityGroup`,
      data: {
        lGrpId,
        pGrpParams
      },
      connection
    })
      .then(this.getResult)
  }
}
